#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Antenna / accesssory switch remote control

(c) dg7bbp 2021-2022

litte bit more secure then pure eval of parsing boolexpressions 
"""

import ast

class BoolParser(object):
    allowedObjects = [ast.Expression,
                      ast.BoolOp,
                      ast.NameConstant,
                      ast.Name,
                      ast.UnaryOp,
                      ast.Compare,
                      ast.Gt,
                      ast.GtE,
                      ast.Lt,
                      ast.LtE,
                      ast.Eq,
                      ast.NotEq,
                      ast.And,
                      ast.Or,
                      ast.Not,
                      ast.Load,
                      ast.Constant,
                      ast.BinOp,
                      ast.Num]
    
    @classmethod
    def is_valid(cls, ast_tree):
        variables = set()
        tree_type = type(ast_tree)
        if  tree_type in cls.allowedObjects:
            if tree_type == ast.Name:
                variables.add(ast_tree.id)
            valid = True
            for o in ast.iter_child_nodes(ast_tree):
                valid, namevars =  cls.is_valid(o)
                if not valid:
                    break
                else:
                    variables.update(namevars)
        else:
            valid = False
        return valid, variables
    
    def __init__(self, str_expr):
        self.expression = str_expr
        self.ast_tree = ast.parse(self.expression, mode="eval")
        
    def get_variables(self):
        return self.is_valid(self.ast_tree)

    def eval(self, var_values):
        return eval(compile(self.ast_tree,"<ast>", mode="eval"),{},var_values)
        

def test():
    b = BoolParser("(f > 144e6 and 146000000 > f) and not PTT and  SW1")
    ok, mvars =  b.get_variables()
    assert ok
    assert mvars == {"f","PTT", "SW1"}
    assert b.eval({"f": 145e6,
                  "PTT": False,
                  "SW1": True})
    assert not b.eval({"f": 145e6,
                       "PTT": False,
                       "SW1": False})
    
    b = BoolParser("(f > 144e6 and 146000000 > f) and not PTT and  SW1 * 10")
    okMult, mvars =  b.get_variables()
    assert not okMult
    
    b = BoolParser("(f > 144e6 and 146000000 > f) and not PTT and  os.path.isfile(r'c:\bin')")
    okFunc, mvars =  b.get_variables()
    assert not okFunc
    

if __name__ == "__main__":
    test()

